@@ -13,7 +13,7 @@
 #' @param samples Default `100`. The maximum number of samples for each pixel. If this is a length-2
 #' vector and the `sample_method` is `stratified`, this will control the number of strata in each dimension.
 #' The total number of samples in this case will be the product of the two numbers.
-#' @param preview Default `TRUE`. Whether to display a real-time progressive preview of the render. Press ESC to cancel the render.
+#' @param preview Default `interactive()`. Whether to display a real-time progressive preview of the render. Press ESC to cancel the render.
 #' @param interactive Default `interactive()`. Whether the scene preview should be interactive. Camera movement orbits around the 
 #' lookat point (unless the mode is switched to free flying), with the following control mapping:
 #' W = Forward, S = Backward, A = Left, D = Right, Q = Up, Z = Down, 
@@ -32,6 +32,11 @@
 #' the aperture and field of view cannot be changed from their initial settings. Additionally,
 #' clicking to direct the camera at the background environment image while using a realistic camera will
 #' not always point to the exact position selected. 
+#' @param denoise Default `TRUE`. Whether to de-noise the final image and preview images. Note, this requires 
+#' the free Intel Open Image Denoise (OIDN) library be installed on your system. Pre-compiled binaries can be installed from
+#' ppenimagedenoise.org, as well as . Linking during rayrender installation is done by defining the environment variable
+#' OIDN_PATH (set it in the .Renviron file by calling `usethis::edit_r_environ()`) to the top-level directory for OIDN (the directory containing the "lib", "bin", and "include"
+#' directories) and re-installing this package from source.
 #' @param camera_description_file Default `NA`. Filename of a camera description file for rendering with
 #' a realistic camera. Several camera files are built-in: `"50mm"`,`"wide"`,`"fisheye"`, and `"telephoto"`.
 #' @param camera_scale Default `1`. Amount to scale the camera up or down in size. Use this rather than scaling a 
@@ -39,7 +44,7 @@
 #' @param iso Default `100`. Camera exposure.
 #' @param film_size Default `22`, in `mm` (scene units in `m`. Size of the film if using a realistic camera, otherwise
 #' ignored.
-#' @param min_variance Default `0.00005`. Minimum acceptable variance for a block of pixels for the 
+#' @param min_variance Default `0`. Minimum acceptable variance for a block of pixels for the 
 #' adaptive sampler. Smaller numbers give higher quality images, at the expense of longer rendering times.
 #' If this is set to zero, the adaptive sampler will be turned off and the renderer
 #' will use the maximum number of samples everywhere.
@@ -95,6 +100,9 @@
 #' @param transparent_background Default `FALSE`. If `TRUE`, any initial camera rays that escape the scene
 #' will be marked as transparent in the final image. If for a pixel some rays escape and others hit a surface,
 #' those pixels will be partially transparent. 
+#' @param integrator_type Default `"rtiow"` (the algorithm specified in the book "Raytracing in One Weekend", a basic
+#' form of path guiding). Other options include `"nee"` (Next Event Estimation, with direct light sampling) 
+#' and `"basic"` (basic pathtracing, for high sample reference renders and debugging only).
 #' @param debug_channel Default `none`. If `depth`, function will return a depth map of rays into the scene 
 #' instead of an image. If `normals`, function will return an image of scene normals, mapped from 0 to 1.
 #' If `uv`, function will return an image of the uv coords. If `variance`, function will return an image 
@@ -108,9 +116,11 @@
 #' @param bvh_type Default `"sah"`, "surface area heuristic". Method of building the bounding volume
 #' hierarchy structure used when rendering. Other option is "equal", which splits tree into groups
 #' of equal size.
-#' @param progress Default `TRUE` if interactive session, `FALSE` otherwise. 
+#' @param progress Default `interactive()` if interactive session, `FALSE` otherwise. 
 #' @param verbose Default `FALSE`. Prints information and timing information about scene
 #' construction and raytracing progress.
+#' @param print_debug_info Default `FALSE`. This will print out additional information on the compilation environment with
+#' each render.
 #' @param new_page Default `TRUE`. Whether to call `grid::grid.newpage()` when plotting the image (if
 #' no filename specified). Set to `FALSE` for faster plotting (does not affect render time).
 #' @export
@@ -123,19 +133,19 @@
 #' if (run_documentation()) {
 #'   scene = generate_ground(depth = -0.5, 
 #'                           material = diffuse(color = "white", checkercolor = "darkgreen"))
-#'   render_scene(scene, parallel = TRUE, samples = 128, sample_method = "sobol")
+#'   render_scene(scene, parallel = TRUE, samples = 16, sample_method = "sobol")
 #' }
 #' if (run_documentation()) {
 #'   # Add a sphere to the center
 #'   scene = scene %>%
 #'     add_object(sphere(x = 0, y = 0, z = 0, radius = 0.5, material = diffuse(color = c(1, 0, 1))))
-#'   render_scene(scene, fov = 20, parallel = TRUE, samples = 128)
+#'   render_scene(scene, fov = 20, parallel = TRUE, samples = 16)
 #' }
 #' if (run_documentation()) {
 #'   # Add a marbled cube
 #'   scene = scene %>%
 #'     add_object(cube(x = 1.1, y = 0, z = 0, material = diffuse(noise = 3)))
-#'   render_scene(scene, fov = 20, parallel = TRUE, samples = 128)
+#'   render_scene(scene, fov = 20, parallel = TRUE, samples = 16)
 #' }
 #' if (run_documentation()) {
 #'   # Add a metallic gold sphere, using stratified sampling for a higher quality render
@@ -144,7 +154,7 @@
 #'     add_object(sphere(x = -1.1, y = 0, z = 0, radius = 0.5, 
 #'                       material = metal(color = "gold", fuzz = 0.1))) %>%
 #'     add_object(sphere(y=10,z=13,radius=2,material=light(intensity=40)))
-#'   render_scene(scene, fov = 20, parallel = TRUE, samples = 128)
+#'   render_scene(scene, fov = 20, parallel = TRUE, samples = 16)
 #' }
 #' if (run_documentation()) {
 #'   # Lower the number of samples to render more quickly (here, we also use only one core).
@@ -160,7 +170,7 @@
 #'   scene = scene %>%
 #'     add_object(xy_rect(x = 0, y = 1.1, z = 0, xwidth = 2, angle = c(0, 0, 0), 
 #'                        material = diffuse(image_texture = image_array)))
-#'   render_scene(scene, fov = 20, parallel = TRUE, samples = 128)
+#'   render_scene(scene, fov = 20, parallel = TRUE, samples = 16)
 #' }
 #' if (run_documentation()) {
 #'   # Move the camera
@@ -171,12 +181,12 @@
 #'   render_scene(scene, lookfrom = c(7, 1.5, 10), lookat = c(0, 0.5, 0), fov = 15,
 #'                backgroundhigh = "orange", backgroundlow = "red", parallel = TRUE,
 #'                ambient = TRUE,
-#'                samples = 128)
+#'                samples = 16)
 #' }
 #' if (run_documentation()) {    
 #'   # Increase the aperture to blur objects that are further from the focal plane.
 #'   render_scene(scene, lookfrom = c(7, 1.5, 10), lookat = c(0, 0.5, 0), fov = 15,
-#'                aperture = 1, parallel = TRUE, samples = 128)
+#'                aperture = 1, parallel = TRUE, samples = 16)
 #' }
 #' if (run_documentation()) {
 #'   # We can also capture a 360 environment image by setting `fov = 360` (can be used for VR)
@@ -193,7 +203,7 @@
 #'                        material = glossy(checkercolor = "white",
 #'                                          checkerperiod = 10, color = "dodgerblue"))) %>%
 #'     render_scene(lookfrom = c(278, 278, 30), lookat = c(278, 278, 500), clamp_value = 10,
-#'                  fov = 360,  samples = 128, width = 800, height = 800)
+#'                  fov = 360,  samples = 16, width = 800, height = 800)
 #' }
 #' if (run_documentation()) {            
 #'   # Spin the camera around the scene, decreasing the number of samples to render faster. To make 
@@ -214,8 +224,9 @@
 render_scene = function(scene, width = 400, height = 400, fov = 20, 
                         samples = 100,  camera_description_file = NA, 
                         preview = interactive(), interactive = TRUE,
+                        denoise = TRUE,
                         camera_scale = 1, iso = 100, film_size = 22,
-                        min_variance = 0.00005, min_adaptive_size = 8,
+                        min_variance = 0, min_adaptive_size = 8,
                         sample_method = "sobol_blue", 
                         max_depth = NA, roulette_active_depth = 100,
                         ambient_light = NULL, 
@@ -227,8 +238,31 @@ render_scene = function(scene, width = 400, height = 400, fov = 20,
                         environment_light = NULL, rotate_env = 0, intensity_env = 1,
                         transparent_background = FALSE,
                         debug_channel = "none", return_raw_array = FALSE, 
-                        progress = interactive(), verbose = FALSE, new_page = TRUE) { 
+                        progress = interactive(), verbose = FALSE, print_debug_info = FALSE,
+                        new_page = TRUE,
+                        integrator_type = "rtiow") { 
   init_time()
+  if(print_debug_info) {
+    message(sprintf(
+"------Debug Info------
+RAY_HAS_X11: %s 
+HAS_SSE: %s 
+HAS_SSE2: %s 
+HAS_SSE3: %s 
+HAS_SSE41:  %s
+HAS_NEON: %s
+HAS_OIDN: %s
+----------------------", 
+      ifelse(has_gui_capability(),"TRUE","FALSE"),
+      ifelse(cppdef_HAS_SSE(),"TRUE","FALSE"),
+      ifelse(cppdef_HAS_SSE2(),"TRUE","FALSE"),
+      ifelse(cppdef_HAS_SSE3(),"TRUE","FALSE"),
+      ifelse(cppdef_HAS_SSE41(),"TRUE","FALSE"),
+      ifelse(cppdef_HAS_NEON(),"TRUE","FALSE"),
+      ifelse(cppdef_HAS_OIDN(),"TRUE","FALSE")
+      )
+    )
+  }
   if(samples > 256 && sample_method == "sobol_blue") {
     warning('"sobol_blue" sample method only valid for `samples` than or equal to 256--switching to `sample_method = "sobol"`')
     sample_method = "sobol"
@@ -294,7 +328,9 @@ Left Mouse Click: Change Look At (new focal distance) | Right Mouse Click: Chang
                                   environment_light = environment_light, rotate_env = rotate_env, 
                                   intensity_env = intensity_env,
                                   debug_channel = debug_channel, return_raw_array = return_raw_array,
-                                  progress = progress, verbose = verbose, sample_dist = Inf)
+                                  progress = progress, verbose = verbose, sample_dist = Inf, 
+                                  integrator_type = integrator_type, denoise = denoise,
+                                  print_debug_info = print_debug_info)
   print_time(verbose, "Pre-processed scene")
   
   camera_info = scene_list$camera_info
