@@ -1,6 +1,7 @@
 #include "PreviewDisplay.h"
 #include "mathinline.h"
 #include "Rcpp.h"
+#include "raylog.h"
 
 
 #ifdef RAY_HAS_X11
@@ -64,11 +65,21 @@ void PreviewDisplay::DrawImage(adaptive_sampler& adaptive_pixel_sampler,
                                size_t &ns, RProgress::RProgress &pb, bool progress,
                                Float percent_done,
                                hitable *world, random_gen& rng) {
+  SCOPED_CONTEXT("Overall");
+  SCOPED_TIMER_COUNTER("Draw Image");
 #ifdef RAY_HAS_X11
   if (d) {
-    RayMatrix &r  = adaptive_pixel_sampler.r;
-    RayMatrix &g  = adaptive_pixel_sampler.g;
-    RayMatrix &b  = adaptive_pixel_sampler.b;
+#ifdef HAS_OIDN
+    if(denoise) {
+      filter.execute();
+    }
+    RayMatrix &rgb  = adaptive_pixel_sampler.draw_rgb_output;
+    if(!denoise) {
+      rgb = adaptive_pixel_sampler.rgb;
+    }
+#else
+    RayMatrix &rgb  = adaptive_pixel_sampler.rgb;
+#endif
     std::vector<bool>& finalized = adaptive_pixel_sampler.finalized;
     std::vector<bool>& just_finalized = adaptive_pixel_sampler.just_finalized;
     
@@ -83,15 +94,15 @@ void PreviewDisplay::DrawImage(adaptive_sampler& adaptive_pixel_sampler,
             g_col = 1;
             b_col = 0;
           } else {
-            r_col = interactive ? std::sqrt((r(i/4,height-1-j))) : std::sqrt((r(i/4,height-1-j))/4);
-            g_col = interactive ? std::sqrt((g(i/4,height-1-j))) : std::sqrt((g(i/4,height-1-j))/4);
-            b_col = interactive ? std::sqrt((b(i/4,height-1-j))) : std::sqrt((b(i/4,height-1-j))/4);
+            r_col = interactive ? std::sqrt((rgb(i/4,height-1-j,0))) : std::sqrt((rgb(i/4,height-1-j,0))/4);
+            g_col = interactive ? std::sqrt((rgb(i/4,height-1-j,1))) : std::sqrt((rgb(i/4,height-1-j,1))/4);
+            b_col = interactive ? std::sqrt((rgb(i/4,height-1-j,2))) : std::sqrt((rgb(i/4,height-1-j,2))/4);
           }
         } else {
           samples = ns+1;
-          r_col = std::sqrt((r(i/4,height-1-j))/samples);
-          g_col = std::sqrt((g(i/4,height-1-j))/samples);
-          b_col = std::sqrt((b(i/4,height-1-j))/samples);
+          r_col = std::sqrt((rgb(i/4,height-1-j,0))/samples);
+          g_col = std::sqrt((rgb(i/4,height-1-j,1))/samples);
+          b_col = std::sqrt((rgb(i/4,height-1-j,2))/samples);
         }
 
         data[i + 4*width*j]   = (unsigned char)(255*clamp(b_col,0,1));
@@ -282,7 +293,7 @@ void PreviewDisplay::DrawImage(adaptive_sampler& adaptive_pixel_sampler,
           if (e.xkey.keycode == P_key || e.xkey.keycode == K_key ) {
             point3f origin = cam->get_origin();
             Float fov =  cam->get_fov();
-            point3f cam_direction = -cam->get_w();
+            vec3f cam_direction = -cam->get_w();
             Float cam_aperture = cam->get_aperture();
             Float fd = cam->get_focal_distance();
             vec3f cam_up = cam->get_up();
@@ -462,7 +473,7 @@ void PreviewDisplay::DrawImage(adaptive_sampler& adaptive_pixel_sampler,
             if (e.xkey.keycode == P_key ) {
               point3f origin = cam->get_origin();
               Float fov =  cam->get_fov();
-              point3f cam_direction = -cam->get_w();
+              vec3f cam_direction = -cam->get_w();
               Float fd = cam->get_focal_distance();
               point3f cam_lookat = cam->get_lookat();
               Float cam_aperture = cam->get_aperture();
@@ -516,11 +527,11 @@ void PreviewDisplay::DrawImage(adaptive_sampler& adaptive_pixel_sampler,
             cam->GenerateRay(samp,&r2);
             if(world->hit(r2, 0.001, FLT_MAX, hrec, rng)) {
               if( hrec.shape->GetName() != "EnvironmentLight") {
-                dir = -hrec.p;
+                dir = point3f(0)-hrec.p;
               }  else {
                 left = false;
                 right = true;
-                dir = -hrec.p;
+                dir = point3f(0)-hrec.p;
               }
             }
           } else if (fov > 0) {
@@ -578,9 +589,12 @@ void PreviewDisplay::DrawImage(adaptive_sampler& adaptive_pixel_sampler,
     pb_w = &pb;
     progress_w = progress;
     interactive_w = interactive;
-    RayMatrix &r  = adaptive_pixel_sampler.r;
-    RayMatrix &g  = adaptive_pixel_sampler.g;
-    RayMatrix &b  = adaptive_pixel_sampler.b;
+#ifdef HAS_OIDN
+    filter.execute();
+    RayMatrix &rgb_s  = adaptive_pixel_sampler.draw_rgb_output;
+#else
+    RayMatrix &rgb_s  = adaptive_pixel_sampler.rgb;
+#endif
     EnvWorldToObject_w = EnvWorldToObject;
     EnvObjectToWorld_w = EnvObjectToWorld;
     Start_EnvWorldToObject_w = Start_EnvWorldToObject;
@@ -590,8 +604,8 @@ void PreviewDisplay::DrawImage(adaptive_sampler& adaptive_pixel_sampler,
     write_fast_output_w = &write_fast_output;
     world_w = world;
     rng_w = &rng;
-    height = (unsigned int)r.cols();
-    width = (unsigned int)r.rows();
+    height = (unsigned int)rgb_s.cols();
+    width = (unsigned int)rgb_s.rows();
     rgb.resize(width*height*3);
     for(unsigned int i = 0; i < width*3; i += 3) {
       for(unsigned int j = 0; j < height; j++) {
@@ -603,15 +617,18 @@ void PreviewDisplay::DrawImage(adaptive_sampler& adaptive_pixel_sampler,
             g_col = 1.f;
             b_col = 0.f;
           } else {
-            r_col = interactive ? std::sqrt((r(i/3,height-1-j))) : std::sqrt((r(i/3,height-1-j))/4.f);
-            g_col = interactive ? std::sqrt((g(i/3,height-1-j))) : std::sqrt((g(i/3,height-1-j))/4.f);
-            b_col = interactive ? std::sqrt((b(i/3,height-1-j))) : std::sqrt((b(i/3,height-1-j))/4.f);
+            r_col = interactive ? std::sqrt((rgb_s(i/3,height-1-j,0))) : 
+              std::sqrt((rgb_s(i/3,height-1-j,0))/4.f);
+            g_col = interactive ? std::sqrt((rgb_s(i/3,height-1-j,1))) : 
+              std::sqrt((rgb_s(i/3,height-1-j,1))/4.f);
+            b_col = interactive ? std::sqrt((rgb_s(i/3,height-1-j,2))) : 
+              std::sqrt((rgb_s(i/3,height-1-j,2))/4.f);
           }
         } else {
           samples = (Float)ns+1.f;
-          r_col = std::sqrt((r(i/3,height-1-j))/samples);
-          g_col = std::sqrt((g(i/3,height-1-j))/samples);
-          b_col = std::sqrt((b(i/3,height-1-j))/samples);
+          r_col = std::sqrt((rgb_s(i/3,height-1-j,0))/samples);
+          g_col = std::sqrt((rgb_s(i/3,height-1-j,1))/samples);
+          b_col = std::sqrt((rgb_s(i/3,height-1-j,2))/samples);
         }
         rgb[i+3*width*j]   = clamp(r_col,0.f,1.f);
         rgb[i+3*width*j+1] = clamp(g_col,0.f,1.f);
@@ -643,21 +660,32 @@ void PreviewDisplay::DrawImage(adaptive_sampler& adaptive_pixel_sampler,
   }
 #endif
 }
-
+#ifdef HAS_OIDN
+PreviewDisplay::PreviewDisplay(unsigned int _width, unsigned int _height, 
+                               bool preview, bool _interactive,
+                               Float initial_lookat_distance, RayCamera* _cam,
+                               Transform* _EnvObjectToWorld, Transform* _EnvWorldToObject, 
+                               oidn::FilterRef& _filter,
+                               bool denoise) :
+  preview(preview), EnvObjectToWorld(_EnvObjectToWorld), EnvWorldToObject(_EnvWorldToObject),
+  Start_EnvObjectToWorld(*_EnvObjectToWorld), Start_EnvWorldToObject(*_EnvWorldToObject), filter(_filter),
+  denoise(denoise) {
+#else
 PreviewDisplay::PreviewDisplay(unsigned int _width, unsigned int _height, 
                                bool preview, bool _interactive,
                                Float initial_lookat_distance, RayCamera* _cam,
                                Transform* _EnvObjectToWorld, Transform* _EnvWorldToObject) :
   preview(preview), EnvObjectToWorld(_EnvObjectToWorld), EnvWorldToObject(_EnvWorldToObject),
   Start_EnvObjectToWorld(*_EnvObjectToWorld), Start_EnvWorldToObject(*_EnvWorldToObject) {
+#endif
   Keyframes.clear();
   write_fast_output = false;
+  terminate = false;
 #ifdef RAY_HAS_X11
   speed = 1.f;
   interactive = _interactive;
-  orbit = true;
-  terminate = false;
   env_y_angle = 0;
+  orbit = true;
   base_step = initial_lookat_distance/20;
   cam = _cam;
   if(preview) {
@@ -704,8 +732,6 @@ PreviewDisplay::PreviewDisplay(unsigned int _width, unsigned int _height,
 #ifdef RAY_WINDOWS
   speed = 1.f;
   interactive = _interactive;
-  orbit = true;
-  terminate = false;
   term = false;
   env_y_angle = 0;
   Keyframes_w = &Keyframes;
@@ -972,7 +998,7 @@ LRESULT CALLBACK WindowProc(HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
         case VK_KEY_P: {
           point3f origin = cam_w->get_origin();
           Float fov =  cam_w->get_fov();
-          point3f cam_direction = -cam_w->get_w();
+          vec3f cam_direction = -cam_w->get_w();
           Float cam_aperture = cam_w->get_aperture();
           Float fd = cam_w->get_focal_distance();
           point2f ortho = cam_w->get_ortho();
@@ -1024,7 +1050,7 @@ LRESULT CALLBACK WindowProc(HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
           case VK_KEY_K: {
             point3f origin = cam_w->get_origin();
             Float fov =  cam_w->get_fov();
-            point3f cam_direction = -cam_w->get_w();
+            vec3f cam_direction = -cam_w->get_w();
             Float cam_aperture = cam_w->get_aperture();
             Float fd = cam_w->get_focal_distance();
             vec3f cam_up = cam_w->get_up();
@@ -1130,10 +1156,8 @@ LRESULT CALLBACK WindowProc(HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
           ray r2;
           cam_w->GenerateRay(samp,&r2);
           if(world_w->hit(r2, 0.001, FLT_MAX, hrec, *rng_w)) {
-            if( hrec.shape->GetName() != "EnvironmentLight") {
-              dir = -hrec.p;
-            }  else {
-              dir = -hrec.p;
+            dir = convert_to_vec3(-hrec.p);
+            if( hrec.shape->GetName() == "EnvironmentLight") {
               just_direction = true;
             }
           }
@@ -1192,11 +1216,7 @@ LRESULT CALLBACK WindowProc(HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
         ray r2;
         cam_w->GenerateRay(samp,&r2);
         if(world_w->hit(r2, 0.001, FLT_MAX, hrec, *rng_w)) {
-          if( hrec.shape->GetName() != "EnvironmentLight") {
-            dir = -hrec.p;
-          }  else {
-            dir = -hrec.p;
-          }
+          dir = convert_to_vec3(-hrec.p);
         }
       } else if (fov > 0) {
         ray r2 = cam_w->get_ray(u,1-v, point3f(0),
