@@ -3,8 +3,9 @@
 
 #include "trianglemesh.h"
 #include "triangle.h"
-#include "bvh_node.h"
+
 #include "rng.h"
+#include "bvh.h"
 #ifndef STBIMAGEH
 #define STBIMAGEH
 #include "stb/stb_image.h"
@@ -25,11 +26,13 @@ public:
           bool displacement_vector, TextureCache& texCache, bool recalculate_normals,
           hitable_list& imp_sample_objects,
           Float shutteropen, Float shutterclose, int bvh_type, random_gen rng, bool verbose,
-          std::shared_ptr<Transform> ObjectToWorld, std::shared_ptr<Transform> WorldToObject, bool reverseOrientation);
-  
-  virtual bool hit(const ray& r, Float t_min, Float t_max, hit_record& rec, random_gen& rng);
-  virtual bool hit(const ray& r, Float t_min, Float t_max, hit_record& rec, Sampler* sampler);
+          Transform* ObjectToWorld, Transform* WorldToObject, bool reverseOrientation);
   
+  virtual const bool hit(const ray& r, Float t_min, Float t_max, hit_record& rec, random_gen& rng) const;
+  virtual const bool hit(const ray& r, Float t_min, Float t_max, hit_record& rec, Sampler* sampler) const;
+  virtual bool HitP(const ray &r, Float t_min, Float t_max, random_gen& rng) const;
+  virtual bool HitP(const ray &r, Float t_min, Float t_max, Sampler* sampler) const;
+
   Float pdf_value(const point3f& o, const vec3f& v, random_gen& rng, Float time = 0);
   Float pdf_value(const point3f& o, const vec3f& v, Sampler* sampler, Float time = 0);
   vec3f random(const point3f& o, random_gen& rng, Float time = 0);
@@ -40,6 +43,11 @@ public:
     return(std::string("TriangleMesh"));
   }
   size_t GetSize();
+  virtual void hitable_info_bounds(Float t0, Float t1) const {
+    aabb box;
+    bounding_box(t0, t1, box);
+    Rcpp::Rcout << GetName() << ": " <<  box.min() << "-" << box.max() << "\n";
+  }
   std::pair<size_t,size_t> CountNodeLeaf();
   
   //Data Members
@@ -47,8 +55,7 @@ public:
   hitable_list triangles;
 
   //Hitable extras
-  std::shared_ptr<material> mat_ptr;
-  std::shared_ptr<bvh_node> tri_mesh_bvh;
+  std::shared_ptr<BVHAggregate> tri_mesh_bvh;
 };
 
 
